(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * PolicyCore.thy ---
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Univerisite Paris-Sud, France 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: PolicyCore.thy 8852 2010-03-24 11:31:11Z lukasbru $ *)

theory PolicyCore
imports NetworkCore
begin

text{* Next, we define the concept of a policy. From an abstract point of view, a
  policy is a partial mapping of packets to decisions. Thus, we model
  the decision as a datatype.
*}
datatype '\<alpha> out = accept '\<alpha> | deny '\<alpha>

text{* 
  A policy is seen as a partial mapping from packet to packet
  out. *}

types ('\<alpha>, '\<beta>) Policy = "('\<alpha>, '\<beta>) packet \<rightharpoonup> (('\<alpha>, '\<beta>) packet) out" 


text{*
  When combining several rules, the firewall is supposed to apply the
  first matching one. In our setting this means the first rule which
  maps the packet in question to @{text "Some (packet out)"}. This is
  exactly what happens when using the map-add operator (@{text "rule1
  ++ rule2"}). The only difference is that the rules must be given in
  reverse order. 
*}



text{*   
  The constant @{text p_accept} is @{text "True"} iff the policy
  accepts the packet. 
*}

definition
  p_accept :: "('\<alpha>, '\<beta>) packet \<Rightarrow> ('\<alpha>, '\<beta>) Policy \<Rightarrow> bool" where
  "p_accept p policy \<equiv>  policy p = Some (accept p)"

end
