(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * TestGen.thy --- the core of the HOL-TestGen System.
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Univerisite Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id:$ *)

header {* The TestGen Core*}

theory TestGen imports SMT 
  uses  "kernel_ext/isabelle2009_kernel_patch" 
	"TestEnv.ML"
        "TestGen.ML" 
        "isar_setup.ML" 
begin 

(* setup "TestGen_setup" *)

setup "TestingIsar_setup"

section{*General Framework for Unit-Test*}

constdefs         THYP   :: "bool \<Rightarrow> bool"
                 "THYP x \<equiv> x"
                  RSF    :: "bool"
                 "RSF    \<equiv> True" (* just to denote random solver failures *)

lemma  THYP_triv [simp]: "THYP True = True" by(simp add: THYP_def)

lemma  THYP_I    : "A \<Longrightarrow> THYP A" by(simp add: THYP_def)

lemma  THYP_D    : "THYP A \<Longrightarrow> A" by(simp add: THYP_def) 

lemma  THYP_E    : "\<lbrakk> THYP A; (A \<Longrightarrow> R) \<rbrakk> \<Longrightarrow> R" by(simp add: THYP_def)

lemma  THYP_spec : "THYP(\<forall>x. P x) \<Longrightarrow> THYP(P x)" by(simp add: THYP_def)

lemma  THYP_app1 : "\<lbrakk> THYP(A \<longrightarrow> B); A \<rbrakk> \<Longrightarrow> B" by(simp add: THYP_def) 
       (* to be used with etac in order to apply uniformity hyp *)

lemma  THYP_app1_rev : "\<lbrakk> A; THYP(A \<longrightarrow> B) \<rbrakk> \<Longrightarrow> B" by(simp add: THYP_def) 
       (* to be used with etac in order to apply uniformity hyp *)

lemma  THYP_app2 : "\<lbrakk> THYP(A); A \<Longrightarrow> B \<rbrakk> \<Longrightarrow> B" by(simp add: THYP_def) 
       (* to be used with RS in order to apply regularity hyp *)


lemma  THYP_E_reg :  "\<lbrakk> size x \<le> k \<Longrightarrow> B; THYP((k < size x) \<longrightarrow> B)\<rbrakk> \<Longrightarrow> B" 
       by(simp add: THYP_def, auto)
       (* to be used with rtac  in order to introduce regularity hyp *)


lemma  RSF_E : "\<lbrakk> RSF \<Longrightarrow> P \<rbrakk> \<Longrightarrow> P"
       by(auto simp: RSF_def)
consts_code  "RSF" ("TestHarness.rsf")

theorem eqTrueI2 :"[|P;P |] ==> True" by auto

section {* ISAR Interface Setup *}

text {* Method for generating test cases *}
setup {*
let 
  fun parse_n_exec (args :  Method.src) (ctx : Proof.context) =  
  let 
     val _              = LogThy.start ()
     val thy            = ProofContext.theory_of ctx;
     val te             = TestGen_DataManagement.get(Context.Theory thy);
     val default_depth  = TestEnv.get_depth te;
     val default_breadth= TestEnv.get_breadth te;
     fun te' n m        = TestEnv.breadth_update n (TestEnv.depth_update m te)
     fun clasimp_meth tac prems ctxt = Method.METHOD (fn facts =>
                                          ALLGOALS (Method.insert_tac (prems @ facts)) 
                                          THEN tac (local_clasimpset_of ctxt));
     fun clasimp_meth' tac prems ctxt = Method.METHOD (fn facts =>
                                           HEADGOAL (Method.insert_tac (prems @ facts) 
                                           THEN' tac (local_clasimpset_of ctxt)));
     val arg_parse = (Scan.lift (Scan.option (OuterParse.nat -- OuterParse.nat) -- Args.name))
     fun gen_test_case_meth (NONE,prog) = 
                         gen_test_case_meth(SOME(default_depth,default_breadth),prog)
       | gen_test_case_meth ((SOME(m, n),prog)) = 
                         clasimp_meth (CHANGED_PROP o 
                                              (fn css => TestGen.gen_test_case_tac ctx css (te' n m) prog));
     val gen_test_case_args  = Method.bang_sectioned_args' clasimp_modifiers arg_parse;
  in 
      gen_test_case_args gen_test_case_meth args ctx
  end

in 
   Method.add_method("gen_test_cases", parse_n_exec, "generate symbolic test cases")
end

*} 



(* setup setup_TestGen *)

end
